package com.sxtvs.open.api.clue.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sxtvs.open.api.clue.dto.ClueDTO;
import com.sxtvs.open.api.clue.dto.VerifyDTO;
import com.sxtvs.open.api.clue.entity.Clue;
import com.sxtvs.open.api.clue.mapper.ClueMapper;
import com.sxtvs.open.api.clue.service.IClueService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sxtvs.open.core.advice.BizException;
import org.apache.http.util.TextUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-03-20
 */
@Service
public class ClueServiceImpl extends ServiceImpl<ClueMapper, Clue> implements IClueService {

    public IPage<Clue> cluePage(ClueDTO clueDTO){

        LambdaQueryWrapper<Clue> wrapper = Wrappers.lambdaQuery();

        return baseMapper.selectPage(new Page<>(clueDTO.getPage(), clueDTO.getSize()), wrapper.ge(clueDTO.getStart() != null, Clue::getAddTime, clueDTO.getStart())
                .le(clueDTO.getEnd() != null, Clue::getAddTime, clueDTO.getEnd())
                .like(!TextUtils.isEmpty(clueDTO.getTitle()), Clue::getTitle, "%" + clueDTO.getTitle() + "%")
                .orderByDesc(Clue::getId)
        );
    }

    public void store(Clue clue){
        clue.setId(null);
        clue.setAddTime(LocalDateTime.now());
        clue.setCreateTime(LocalDateTime.now());
        clue.setModifyTime(LocalDateTime.now());
        save(clue);
    }

    public void verify(VerifyDTO verifyDTO){
        Clue clue = getById(verifyDTO.getId());
        if(clue.getStatus() != 0){
            throw new BizException(40001, "该条数据已审核");
        }
        clue.setStatus(verifyDTO.getStatus());
        updateById(clue);
    }
}
