package com.sxtvs.open.api.odata.controller;

import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.sxtvs.open.api.odata.service.*;
import com.sxtvs.open.api.odata.service.impl.WbYmAccountServiceImpl;
import com.sxtvs.open.core.sls.AliyunLogger;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import java.io.IOException;
import java.util.Map;

@Controller
@RequestMapping("oauth")
public class OauthController {


    @Autowired
    private OauthService oauthService;

    @Autowired
    private AliyunLogger logger;

    @Autowired
    private DouyinAccountService douyinAccountService;

    @RequestMapping("douyin/callback")
    public String douyinCallback(String code, String state) {
        var token = oauthService.loadStateMapping(state);
        logger.info("key", "oauth/douyin/callback", "code", code, "state", state, "token", token);

        douyinAccountService.upsert(code);
        return "redirect:https://open.sxtvs.net/oauth/close";
    }

    @Autowired
    private KuaishouAccountService kuaishouAccountService;

    @RequestMapping("kuaishou/callback")
    public String kuaishouCallback(String code, String state) {
        var token = oauthService.loadStateMapping(state);
        logger.info("key", "oauth/kuaishou/callback", "code", code, "state", state, "token", token);

        kuaishouAccountService.upsert(code);
        return "redirect:https://open.sxtvs.net/oauth/close";
    }

    @Autowired
    private ToutiaoAccountService toutiaoAccountService;

    @RequestMapping("toutiao/callback")
    public String toutiaoCallback(String code, String state) {
        var token = oauthService.loadStateMapping(state);
        logger.info("key", "oauth/toutiao/callback", "code", code, "state", state, "token", token);

        toutiaoAccountService.upsert(code);
        return "redirect:https://open.sxtvs.net/oauth/close";
    }

    @Autowired
    private WbYmAccountServiceImpl weiboAccountService;

    @RequestMapping("weibo/callback")
    public String weiboCallback(@RequestParam("network_group_id") String networkGroupId, String state) throws JsonProcessingException {
        logger.info("key", "oauth/weibo/callback", "code", networkGroupId);

        weiboAccountService.upsert(networkGroupId);
        return "redirect:https://open.sxtvs.net/oauth/close";
    }

    @Autowired
    private BilibiliAccountService bilibiliAccountService;

    @RequestMapping("bilibili/callback")
    public String bilibiliCallback(String code, String state) {
        var token = oauthService.loadStateMapping(state);
        logger.info("key", "oauth/bilibili/callback", "code", code, "state", state, "token", token);
        bilibiliAccountService.upsert(code);
        return "redirect:https://open.sxtvs.net/oauth/close";
    }

    @Autowired
    private WxRestService wxRestService;

    @RequestMapping("weixin/{appid}/callback")
    public String callback(
            @RequestParam("auth_code") String authCode,
            @RequestParam("expires_in") Integer expiresIn,
            @PathVariable("appid") String appid, HttpServletRequest request) {
        logger.info("key", "oauth/{appid}/callback", "data",
                Map.of("authCode", authCode, "expiresIn", expiresIn, "appid", appid));
        if (StrUtil.isBlank(authCode)) {
            try {
                String body = IOUtils.toString(request.getReader());
                logger.error("wxdca527f24b99b4c0/callback error body {}", body);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        wxRestService.apiQueryAuth(authCode);
        return "redirect:https://open.sxtvs.net/oauth/close";
    }

    @RequestMapping("close")
    public ModelAndView winClose(){
        ModelAndView modelAndView = new ModelAndView();
        modelAndView.setViewName("close");
        return modelAndView;
    }
}
