package com.sxtvs.open.api.odata.service;

import cn.hutool.core.util.IdUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.odata.dto.wx.WxApiCreatePreAuthCodeRequest;
import com.sxtvs.open.api.odata.dto.wx.WxApiCreatePreAuthCodeResponse;
import com.sxtvs.open.api.youmei.service.YoumeiAccountServiceImpl;
import com.sxtvs.open.core.conf.Constant;
import com.sxtvs.open.core.sls.AliyunLogger;
import lombok.SneakyThrows;
import org.apache.hc.client5.http.fluent.Request;
import org.apache.hc.core5.http.ContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.util.Map;
import java.util.Optional;

@Service
public class OauthService {
    @Autowired
    private YoumeiAccountServiceImpl youmeiAccountService;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private AliyunLogger logger;

    @SneakyThrows
    public String getWeiboUrl() {
        var accessToken = youmeiAccountService.getSxgdAccount().getAccessToken();
        var state = IdUtil.nanoId();
        var body = Request.get("https://api-open.51wyq.cn/dataapp/api/umei/fw/open/network_group/get_auth_url?accessToken=" + accessToken +
                "&state="+state+"&redirect_uri=https://open.sxtvs.net/oauth/weibo/callback")
                .execute()
                .returnContent()
                .asString(StandardCharsets.UTF_8);

        return objectMapper.readTree(body).get("data").get("auth_url").asText();
    }


    @SneakyThrows
    public WxApiCreatePreAuthCodeResponse createPreAuthCodeResponse() {
        var ticket = redisTemplate.opsForValue().get("WX_TICKET");
        WxApiCreatePreAuthCodeRequest authCodeRequest = new WxApiCreatePreAuthCodeRequest();
        authCodeRequest.setComponentAppid(Constant.wxClientKey);
        authCodeRequest.setComponentAccessToken(ticket);
        var body = Request.post("https://api.weixin.qq.com/cgi-bin/component/api_create_preauthcode?" +
                        "component_access_token=" + ticket)
                .bodyString(objectMapper.writeValueAsString(Map.of("component_appid", Constant.wxClientKey)),
                        ContentType.APPLICATION_JSON)
                .execute()
                .returnContent()
                .asString(StandardCharsets.UTF_8);
        logger.info("微信获取preauthcode {}", body);

        return objectMapper.readValue(body, WxApiCreatePreAuthCodeResponse.class);
    }

    public void saveStateMapping(String token, String state) {
        redisTemplate.opsForValue().set("OAUTH_STATE:" + state, token, Duration.ofMinutes(3));
    }

    public Optional<String> loadStateMapping(String state) {
        var token = redisTemplate.opsForValue().get("OAUTH_STATE:" + state);
        return Optional.ofNullable(token);
    }

}
