package com.sxtvs.open.api.odata.service;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.odata.dao.WeiboAccountMapper;
import com.sxtvs.open.api.odata.dto.weibo.WeiboAccessTokenResponse;
import com.sxtvs.open.api.odata.dto.weibo.WeiboCreateTask;
import com.sxtvs.open.api.odata.dto.weibo.show.UserShowResponse;
import com.sxtvs.open.api.odata.entity.WeiboAccount;
import com.sxtvs.open.api.odata.entity.WeiboTask;
import com.sxtvs.open.core.conf.Constant;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

@Service
@Slf4j
public class WeiboAccountService extends ServiceImpl<WeiboAccountMapper, WeiboAccount> {


    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private WeiboTaskService weiboTaskService;

    @Autowired
    private QrcodeLogServiceImpl qrcodeLogService;



    @SneakyThrows
    public void upsert(String code) {
        Map<String, Object> map = new HashMap<>();
        map.put("client_id", Constant.weiboKey);
        map.put("client_secret", Constant.weiboSecret);
        map.put("grant_type", "authorization_code");
        map.put("code", code);
        map.put("redirect_uri", "https://open.sxtvs.net/weibo/callback");

        String response = HttpUtil.post("https://api.weibo.com/oauth2/access_token", map);
        log.info(response);
        WeiboAccessTokenResponse tokenResponse = objectMapper.readValue(response, WeiboAccessTokenResponse.class);
        tokenResponse.setAccessToken("OAuth2 " + tokenResponse.getAccessToken());
        String uid = tokenResponse.getUid();
        map.clear();

        response = HttpUtil.createGet("https://api.weibo.com/2/users/show.json")
                .header("Authorization", tokenResponse.getAccessToken())
                .form("uid", uid)
                .execute()
                .body();
        UserShowResponse userShow = objectMapper.readValue(response, UserShowResponse.class);
        log.info("response {}", response);

        qrcodeLogService.writeLog("微博",uid,userShow.getName());

        WeiboAccount weiboAccount = this.getById(uid);
        DateTime now = new DateTime();
        if (weiboAccount == null) {
            weiboAccount = new WeiboAccount();
            weiboAccount.setOpenId(uid);
            weiboAccount.setAccessToken(tokenResponse.getAccessToken());
            weiboAccount.setAtExpireTime(DateUtil.offsetSecond(now, tokenResponse.getExpiresIn()));
            weiboAccount.setCreateTime(now);
            weiboAccount.setUpdateTime(now);
            weiboAccount.setNickName(userShow.getName());
            this.save(weiboAccount);
        } else {
            weiboAccount.setUpdateTime(now);
            weiboAccount.setAccessToken(tokenResponse.getAccessToken());
            weiboAccount.setAtExpireTime(DateUtil.offsetSecond(now, tokenResponse.getExpiresIn()));
            weiboAccount.setNickName(userShow.getName());
            this.updateById(weiboAccount);
        }
        log.info(tokenResponse.toString());
    }

    @SneakyThrows
    public void createTask(long start, long end, String ids) {
        WeiboAccount weiboAccount = getOne(null);
        String accessToken = weiboAccount.getAccessToken();
        String body = HttpUtil.createPost("https://c.api.weibo.com/2/search/statuses/historical/create.json")
                .header("Authorization", accessToken)
                .form("starttime", start)
                .form("endtime", end)
                .form("ids", ids)
                .execute()
                .body();
        WeiboCreateTask weiboCreateTask = objectMapper.readValue(body, WeiboCreateTask.class);
        if (weiboCreateTask.hasError()) {
            log.error(body);
            return;
        }
        WeiboTask weiboTask = new WeiboTask();
        weiboTask.setTaskId(weiboCreateTask.getTaskId());
        weiboTask.setSecretKey(weiboCreateTask.getSecretKey());
        weiboTask.setFinish(0);
        weiboTask.setStartTime(start);
        weiboTask.setEndTime(end);
        weiboTask.setStartDay(new DateTime(start).toString("yyyyMMdd"));
        weiboTask.setWeiboId(ids);
        weiboTaskService.save(weiboTask);
    }

//    public Data videoList(String openId, String cursor) {
//        WeiboAccount weiboAccount = this.getById(openId);
//        String accessToken = weiboAccount.getAccessToken();
//        HttpUtil.createGet("https://api.weibo.com/2/statuses/repost_timeline.json")
//                .header("Authorization", accessToken);
//        HashMap<String, String> param = new HashMap<String, String>() {{
//            this.put("open_id", weiboAccount.getOpenId());
//            this.put("access_token", weiboAccount.getAccessToken());
//            this.put("cursor", cursor);
//            this.put("count", "10");
//        }};
//        DouyinVideoListResponse douyinVideoListResponse = restTemplate.getForObject(
//                "https://open.douyin.com/toutiao/video/list/?open_id={open_id}&access_token={access_token}&" +
//                        "cursor={cursor}&count={count}",
//                DouyinVideoListResponse.class, param);
//        if (douyinVideoListResponse == null) {
//            throw new RuntimeException();
//        }
//
//        return douyinVideoListResponse.getData();
//    }

}
