package com.sxtvs.open.api.sms.service;


import com.aliyun.dysmsapi20170525.models.SendSmsRequest;
import com.aliyun.dysmsapi20170525.models.SendSmsResponse;
import com.aliyun.teaopenapi.models.Config;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.sms.dao.VerifyCodeMapper;
import com.sxtvs.open.api.sms.entity.VerifyCode;
import com.sxtvs.open.core.advice.BizException;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.RandomStringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Component
public class SmsService {

    @Resource
    private VerifyCodeMapper verifyCodeMapper;

    @Autowired
    private ObjectMapper objectMapper;

    private final Logger log = LoggerFactory.getLogger(this.getClass());

    private com.aliyun.dysmsapi20170525.Client client;

    {
        try {
            client = new com.aliyun.dysmsapi20170525.Client(new Config()
                    // 您的AccessKey ID
                    .setAccessKeyId("LTAI4GEBqfF1GX4VwsYU2Wpg")
                    // 您的AccessKey Secret
                    .setAccessKeySecret("rVIv0E1lRfXOCrAmkFTZnfgWiuv4ea")
                    .setEndpoint("dysmsapi.aliyuncs.com"));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            System.exit(1);
        }
    }

    public void verifyCode(String phone, String code) {
        VerifyCode verifyCode = verifyCodeMapper.selectOne(new LambdaQueryWrapper<VerifyCode>()
                .eq(VerifyCode::getPhone, phone)
                .eq(VerifyCode::getCode, code)
        );
        if (verifyCode == null) {
            throw new BizException("验证码错误");
        }
        if (System.currentTimeMillis() > verifyCode.getExpireTime()) {
            throw new BizException("验证码已超时,请重新发送");
        }
    }


    public void sendVerifyCode(String phone) {
        String pattern = "^1\\d{10}$";
        Pattern p = Pattern.compile(pattern);
        Matcher match = p.matcher(phone);
        if (!match.matches()) {
            throw new BizException(40000, "请输入正确的手机号码");
        }

        VerifyCode verifyCode = verifyCodeMapper.selectOne(new LambdaQueryWrapper<VerifyCode>()
                .eq(VerifyCode::getPhone, phone)
        );
        if (verifyCode != null) {
            if (System.currentTimeMillis() - (verifyCode.getExpireTime() - 60 * 5 * 1000) < 60 * 1000) {
                throw new BizException("验证码已发送,请等待60秒后重新发送");
            }
            verifyCodeMapper.deleteById(phone);
        }


        String code = RandomStringUtils.randomNumeric(4);


        SendSmsRequest sendSmsRequest = new SendSmsRequest()
                .setPhoneNumbers(phone)
                .setSignName("陕西广电融媒体集团")
                .setTemplateCode("SMS_219739949")
                .setTemplateParam("{\"code\":\"" + code + "\"}");

        // 复制代码运行请自行打印 API 的返回值
        try {
            SendSmsResponse sendSmsResponse = client.sendSms(sendSmsRequest);
            log.info("sendSmsResponse {}", objectMapper.writeValueAsString(sendSmsResponse.getBody()));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new BizException("发送短信异常");
        }

        verifyCode = new VerifyCode();
        verifyCode.setPhone(phone);
        verifyCode.setCode(code);
        // 5分钟的过期时间
        verifyCode.setExpireTime(System.currentTimeMillis() + 60 * 5 * 1000);
        verifyCodeMapper.insert(verifyCode);

    }
}
