package com.sxtvs.open.core.auth;




import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.AES;
import com.sxtvs.open.core.advice.BizException;
import com.sxtvs.open.core.conf.Constant;

import java.nio.charset.StandardCharsets;

public class AESUtil {
    private static final AES aes = SecureUtil.aes(Constant.AES_KEY.getBytes(StandardCharsets.UTF_8));

    public static String encryptHex(String data, long addSecond) {
        //加入了随机旋转和时间戳 避免加密后得到的是同样的数据 得到的token比传统的jwt要小很多
        int randomInt = RandomUtil.randomInt(data.length());
        data = shift(data, randomInt);
        return aes.encryptHex(randomInt + "," + data + "," + (getCurrentTime() + addSecond));
    }

    public static String encryptHex(String data){
        return encryptHex(data,Constant.ACCESS_TOKEN_EXPIRE_TIME);
    }

    public static String decryptStr(String token) {
        String[] dataArray;
        long time;
        try {
            token = aes.decryptStr(token, CharsetUtil.CHARSET_UTF_8);
            dataArray = token.split(",");
            time = Long.parseLong(dataArray[2]);
        } catch (Exception e) {
            throw new BizException(Constant.TOKEN_PARSE_ERROR, "token 异常");
        }
        if (time < getCurrentTime()) {
            throw new BizException(Constant.TOKEN_EXPIRE_ERROR, "token 已过期");
        }
        int i = Integer.parseInt(dataArray[0]);
        return shift(dataArray[1], i * -1);
    }

    public static boolean isOk(String token){
        try {
            decryptStr(token);
        } catch (Exception e) {
            return false;
        }
        return true;
    }



    private static long getCurrentTime() {
        return System.currentTimeMillis() / 1000 - Constant.SUB_TIME;
    }

    private static String shift(String str, int n) {
        if (n == 0) {
            return str;
        }
        int len = str.length();
        char[] oldStr = str.toCharArray();
        char[] newStr = new char[len];
        n = n % len;
        if (n < 0) {
            n = len + n;
        }
        for (int i = 0; i < len; i++) {
            newStr[(i + n) % len] = oldStr[i];
        }
        return new String(newStr);
    }



}
