package com.sxtvs.open.core.conf;

import cn.hutool.core.io.IoUtil;
import com.fasterxml.jackson.databind.ObjectMapper;

import com.sxtvs.open.api.odata.dto.douyin.DouyinCommonResponse;
import com.sxtvs.open.api.odata.dto.kuaishou.KuaishouCommonResponse;
import com.sxtvs.open.api.odata.dto.wx.WxCommonResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.BufferingClientHttpRequestFactory;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.web.client.ResponseErrorHandler;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;

@Configuration
@Slf4j
public class RestConfig {

    @Autowired
    private ObjectMapper objectMapper;


    @Bean("douyinRestTemplate")
    public RestTemplate template() {

        RestTemplate restTemplate = getTemplate();

        restTemplate.setErrorHandler(new ResponseErrorHandler() {
            @Override
            public boolean hasError(ClientHttpResponse clientHttpResponse) throws IOException {

                int rawStatusCode = clientHttpResponse.getRawStatusCode();
                String body = IoUtil.read(clientHttpResponse.getBody(), false).toString();
                // 不是200的直接拿去重试
                if (rawStatusCode != 200) {
                    throw new RestClientException(body);
                }
                DouyinCommonResponse douyinCommonResponse = objectMapper.readValue(body,
                        DouyinCommonResponse.class);
                if (douyinCommonResponse.hasError()) {
                    throw new RuntimeException(body);
                }
                return false;
            }

            @Override
            public void handleError(ClientHttpResponse clientHttpResponse) throws IOException {
            }
        });
        return restTemplate;
    }

    @Bean("weixinRestTemplate")
    public RestTemplate weixinTemplate() {

        RestTemplate restTemplate = getTemplate();

        restTemplate.setErrorHandler(new ResponseErrorHandler() {
            @Override
            public boolean hasError(ClientHttpResponse clientHttpResponse) throws IOException {

                int rawStatusCode = clientHttpResponse.getRawStatusCode();
                String body = IoUtil.read(clientHttpResponse.getBody(), false).toString();
                // 不是200的直接拿去重试
                if (rawStatusCode != 200) {
                    throw new RestClientException(body);
                }
                WxCommonResponse douyinCommonResponse = objectMapper.readValue(body,
                        WxCommonResponse.class);
                if (douyinCommonResponse.hasError()) {
                    throw new RuntimeException(body);
                }
                return false;
            }

            @Override
            public void handleError(ClientHttpResponse clientHttpResponse) throws IOException {
            }
        });
        return restTemplate;
    }


    @Bean("kuaishouRestTemplate")
    public RestTemplate kuaishou() {

        RestTemplate restTemplate = getTemplate();

        restTemplate.setErrorHandler(new ResponseErrorHandler() {
            @Override
            public boolean hasError(ClientHttpResponse clientHttpResponse) throws IOException {

                int rawStatusCode = clientHttpResponse.getRawStatusCode();
                String body = IoUtil.read(clientHttpResponse.getBody(), false).toString();
                // 不是200的直接拿去重试
                if (rawStatusCode != 200) {
                    throw new RestClientException(body);
                }
                KuaishouCommonResponse kuaishouCommonResponse = objectMapper.readValue(body,
                        KuaishouCommonResponse.class);
                if (kuaishouCommonResponse.hasError()) {
                    throw new RuntimeException(body);
                }
                return false;
            }

            @Override
            public void handleError(ClientHttpResponse clientHttpResponse) throws IOException {
            }
        });
        return restTemplate;
    }

    private RestTemplate getTemplate() {
        SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
        requestFactory.setConnectTimeout(5000);// 设置超时
        requestFactory.setReadTimeout(5000);
        BufferingClientHttpRequestFactory bufferingClientHttpRequestFactory = new BufferingClientHttpRequestFactory(requestFactory);

        return new RestTemplate(bufferingClientHttpRequestFactory);
    }
}
