package com.sxtvs.open;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch._types.SortOptions;
import co.elastic.clients.elasticsearch._types.SortOrder;
import co.elastic.clients.elasticsearch._types.query_dsl.MultiMatchQuery;
import co.elastic.clients.elasticsearch._types.query_dsl.Query;
import co.elastic.clients.elasticsearch.core.*;
import co.elastic.clients.elasticsearch.core.bulk.BulkOperation;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.news.entity.YoumeiData;
import com.sxtvs.open.api.news.service.impl.YoumeiDataServiceImpl;
import jakarta.annotation.Resource;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@SpringBootTest
public class EsClientTests {

    @Resource
    private ElasticsearchClient elasticsearchClient;

    @Resource
    private YoumeiDataServiceImpl youmeiDataService;

    @Resource
    private ObjectMapper objectMapper;

    @Test
    public void test1(){
        try {
//            elasticsearchClient.indices().delete(c -> c.index("news_data"));
//            elasticsearchClient.indices().create(c -> c.index("news_data"));
            System.out.println(elasticsearchClient.indices().exists(b -> b.index("news_data")).value());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void test2() throws IOException {
        List<YoumeiData> youmeiDataList = youmeiDataService.lambdaQuery().gt(YoumeiData::getOffset, 7773600L).last("limit 20").list();

        YoumeiData d = youmeiDataList.get(0);
        String s = objectMapper.writeValueAsString(d);
        System.out.println(s);
        elasticsearchClient.index(IndexRequest.of(x->{
            x.index("news_data");
            x.id(String.valueOf(d.getOffset()));
            x.document(d);
            return x;
        }));

    }

    @Test
    public void insert() throws IOException, JsonProcessingException {

        List<YoumeiData> youmeiDataList = youmeiDataService.lambdaQuery().gt(YoumeiData::getOffset, 7773600L).last("limit 20").list();

        List<BulkOperation> bulkOperations = new ArrayList<>();

        youmeiDataList.forEach(a -> {

            bulkOperations.add(BulkOperation.of(b -> b.index(c -> c.id(String.valueOf(a.getOffset())).document(a))));
        });

        elasticsearchClient.bulk(BulkRequest.of(x ->x.index("news_data").operations(bulkOperations)));
    }

    @Test
    public void search() throws IOException {

        SearchResponse<YoumeiData> response = elasticsearchClient.search(SearchRequest.of(x -> x.index("news_data").query(
                Query.of(y -> y.multiMatch(MultiMatchQuery.of(z -> z.fields(Arrays.asList("title", "content")).query("拜登为何此时秘密访问乌克兰"))))
        ).size(30)), YoumeiData.class);
        List<YoumeiData> data = new ArrayList<>();
        response.hits().hits().forEach(x -> {
            System.out.println(x.score());
            data.add(x.source());
            System.out.println(x.source());
        });

        System.out.println(data.size());
        System.out.println(data.stream().limit(2));
    }

    @Test
    public void searchPage() throws IOException {

        SearchResponse<YoumeiData> response = elasticsearchClient.search(SearchRequest.of(x -> x.index("news_data").sort(SortOptions.of(
                so -> so.field(fs->fs.field("offset").order(SortOrder.Desc))
        )).size(100).from(0)), YoumeiData.class);
        List<YoumeiData> data = new ArrayList<>();
        response.hits().hits().forEach(x -> {
            System.out.println(x.score());
            data.add(x.source());
            System.out.println(x.source());
        });

        System.out.println(response.hits().total().value());

        System.out.println(data.size());
        System.out.println(data.stream().limit(2));
    }
}
